import os.path
import pathlib
import re
import subprocess
from list_all_files_recursively import get_folder_file_complete_path
from getfilenuitkapython import get_filepath

mimesreversed = {
    "application/vnd.lotus-1-2-3": ["123"],
    "application/vnd.1000minds.decision-model+xml": ["1km"],
    "text/vnd.in3d.3dml": ["3dml"],
    "image/x-3ds": ["3ds"],
    "video/3gpp2": ["3g2"],
    "video/3gpp": ["3gp", "3gpp"],
    "model/3mf": ["3mf"],
    "application/x-7z-compressed": ["7z"],
    "application/x-authorware-bin": ["aab", "u32", "vox", "x32"],
    "audio/x-aac": ["aac"],
    "application/x-authorware-map": ["aam"],
    "application/x-authorware-seg": ["aas"],
    "application/x-abiword": ["abw"],
    "application/vnd.nokia.n-gage.ac+xml": ["ac"],
    "application/vnd.americandynamics.acc": ["acc"],
    "application/x-ace-compressed": ["ace"],
    "application/vnd.acucobol": ["acu"],
    "application/vnd.acucorp": ["acutc", "atc"],
    "audio/adpcm": ["adp"],
    "audio/aac": ["adts"],
    "application/vnd.audiograph": ["aep"],
    "application/x-font-type1": ["afm", "pfa", "pfb", "pfm"],
    "application/vnd.ibm.modcap": ["afp", "list3820", "listafp"],
    "application/vnd.age": ["age"],
    "application/vnd.ahead.space": ["ahead"],
    "application/postscript": ["ai", "eps", "ps"],
    "audio/x-aiff": ["aif", "aifc", "aiff"],
    "application/vnd.adobe.air-application-installer-package+zip": ["air"],
    "application/vnd.dvb.ait": ["ait"],
    "application/vnd.amiga.ami": ["ami"],
    "application/automationml-aml+xml": ["aml"],
    "application/automationml-amlx+zip": ["amlx"],
    "audio/amr": ["amr"],
    "application/vnd.android.package-archive": ["apk"],
    "image/apng": ["apng"],
    "text/cache-manifest": ["appcache", "manifest"],
    "application/appinstaller": ["appinstaller"],
    "application/x-ms-application": ["application"],
    "application/appx": ["appx"],
    "application/appxbundle": ["appxbundle"],
    "application/vnd.lotus-approach": ["apr"],
    "application/x-freearc": ["arc"],
    "application/x-arj": ["arj"],
    "application/pgp-signature": ["asc", "sig"],
    "video/x-ms-asf": ["asf", "asx"],
    "text/x-asm": ["asm", "s"],
    "application/vnd.accpac.simply.aso": ["aso"],
    "application/atom+xml": ["atom"],
    "application/atomcat+xml": ["atomcat"],
    "application/atomdeleted+xml": ["atomdeleted"],
    "application/atomsvc+xml": ["atomsvc"],
    "application/vnd.antix.game-component": ["atx"],
    "audio/basic": ["au", "snd"],
    "image/avci": ["avci"],
    "image/avcs": ["avcs"],
    "video/x-msvideo": ["avi"],
    "image/avif": ["avif"],
    "application/applixware": ["aw"],
    "application/vnd.airzip.filesecure.azf": ["azf"],
    "application/vnd.airzip.filesecure.azs": ["azs"],
    "image/vnd.airzip.accelerator.azv": ["azv"],
    "application/vnd.amazon.ebook": ["azw"],
    "image/vnd.pco.b16": ["b16"],
    "application/x-msdownload": ["bat", "com", "dll", "exe", "msi"],
    "application/x-bcpio": ["bcpio"],
    "application/x-font-bdf": ["bdf"],
    "application/vnd.syncml.dm+wbxml": ["bdm"],
    "application/x-bdoc": ["bdoc"],
    "application/vnd.realvnc.bed": ["bed"],
    "application/vnd.fujitsu.oasysprs": ["bh2"],
    "application/octet-stream": [
        "bin",
        "bpk",
        "buffer",
        "deploy",
        "dist",
        "distz",
        "dms",
        "dump",
        "elc",
        "img",
        "lrf",
        "mar",
        "msm",
        "msp",
        "pkg",
        "so",
    ],
    "application/x-blorb": ["blb", "blorb"],
    "application/vnd.bmi": ["bmi"],
    "application/vnd.balsamiq.bmml+xml": ["bmml"],
    "image/x-ms-bmp": ["bmp"],
    "application/vnd.framemaker": ["book", "fm", "frame", "maker"],
    "application/vnd.previewsystems.box": ["box"],
    "application/x-bzip2": ["boz", "bz2"],
    "model/vnd.valve.source.compiled-map": ["bsp"],
    "image/prs.btif": ["btf", "btif"],
    "application/x-bzip": ["bz"],
    "text/x-c": ["c", "cc", "cpp", "cxx", "dic", "h", "hh"],
    "application/vnd.cluetrust.cartomobile-config": ["c11amc"],
    "application/vnd.cluetrust.cartomobile-config-pkg": ["c11amz"],
    "application/vnd.clonk.c4group": ["c4d", "c4f", "c4g", "c4p", "c4u"],
    "application/vnd.ms-cab-compressed": ["cab"],
    "audio/x-caf": ["caf"],
    "application/vnd.tcpdump.pcap": ["cap", "dmp", "pcap"],
    "application/vnd.curl.car": ["car"],
    "application/vnd.ms-pki.seccat": ["cat"],
    "application/x-cbr": ["cb7", "cba", "cbr", "cbt", "cbz"],
    "application/x-cocoa": ["cco"],
    "application/x-director": [
        "cct",
        "cst",
        "cxt",
        "dcr",
        "dir",
        "dxr",
        "fgd",
        "swa",
        "w3d",
    ],
    "application/ccxml+xml": ["ccxml"],
    "application/vnd.contact.cmsg": ["cdbcmsg"],
    "application/x-netcdf": ["cdf", "nc"],
    "application/cdfx+xml": ["cdfx"],
    "application/vnd.mediastation.cdkey": ["cdkey"],
    "application/cdmi-capability": ["cdmia"],
    "application/cdmi-container": ["cdmic"],
    "application/cdmi-domain": ["cdmid"],
    "application/cdmi-object": ["cdmio"],
    "application/cdmi-queue": ["cdmiq"],
    "chemical/x-cdx": ["cdx"],
    "application/vnd.chemdraw+xml": ["cdxml"],
    "application/vnd.cinderella": ["cdy"],
    "application/pkix-cert": ["cer"],
    "application/x-cfs-compressed": ["cfs"],
    "image/cgm": ["cgm"],
    "application/x-chat": ["chat"],
    "application/vnd.ms-htmlhelp": ["chm"],
    "application/vnd.kde.kchart": ["chrt"],
    "chemical/x-cif": ["cif"],
    "application/vnd.anser-web-certificate-issue-initiation": ["cii"],
    "application/vnd.ms-artgalry": ["cil"],
    "application/node": ["cjs"],
    "application/vnd.claymore": ["cla"],
    "application/java-vm": ["class"],
    "model/vnd.cld": ["cld"],
    "application/vnd.crick.clicker.keyboard": ["clkk"],
    "application/vnd.crick.clicker.palette": ["clkp"],
    "application/vnd.crick.clicker.template": ["clkt"],
    "application/vnd.crick.clicker.wordbank": ["clkw"],
    "application/vnd.crick.clicker": ["clkx"],
    "application/x-msclip": ["clp"],
    "application/vnd.cosmocaller": ["cmc"],
    "chemical/x-cmdf": ["cmdf"],
    "chemical/x-cml": ["cml"],
    "application/vnd.yellowriver-custom-menu": ["cmp"],
    "image/x-cmx": ["cmx"],
    "application/vnd.rim.cod": ["cod"],
    "text/coffeescript": ["coffee", "litcoffee"],
    "text/plain": ["conf", "def", "in", "ini", "list", "log", "text", "txt"],
    "application/x-cpio": ["cpio"],
    "application/cpl+xml": ["cpl"],
    "application/mac-compactpro": ["cpt"],
    "application/x-mscardfile": ["crd"],
    "application/pkix-crl": ["crl"],
    "application/x-x509-ca-cert": ["crt", "der", "pem"],
    "application/x-chrome-extension": ["crx"],
    "application/vnd.rig.cryptonote": ["cryptonote"],
    "application/x-csh": ["csh"],
    "application/vnd.citationstyles.style+xml": ["csl"],
    "chemical/x-csml": ["csml"],
    "application/vnd.commonspace": ["csp"],
    "text/css": ["css"],
    "text/csv": ["csv"],
    "application/cu-seeme": ["cu"],
    "text/vnd.curl": ["curl"],
    "application/cwl": ["cwl"],
    "application/prs.cww": ["cww"],
    "model/vnd.collada+xml": ["dae"],
    "application/vnd.mobius.daf": ["daf"],
    "application/vnd.dart": ["dart"],
    "application/vnd.fdsn.seed": ["dataless", "seed"],
    "application/davmount+xml": ["davmount"],
    "application/vnd.dbf": ["dbf"],
    "application/docbook+xml": ["dbk"],
    "text/vnd.curl.dcurl": ["dcurl"],
    "application/vnd.oma.dd2+xml": ["dd2"],
    "application/vnd.fujixerox.ddd": ["ddd"],
    "application/vnd.syncml.dmddf+xml": ["ddf"],
    "image/vnd.ms-dds": ["dds"],
    "application/x-debian-package": ["deb", "udeb"],
    "application/vnd.dreamfactory": ["dfac"],
    "application/x-dgc-compressed": ["dgc"],
    "image/bmp": ["dib"],
    "application/vnd.mobius.dis": ["dis"],
    "message/disposition-notification": ["disposition-notification"],
    "image/vnd.djvu": ["djv", "djvu"],
    "application/x-apple-diskimage": ["dmg"],
    "application/vnd.dna": ["dna"],
    "application/msword": ["doc", "dot"],
    "application/vnd.ms-word.document.macroenabled.12": ["docm"],
    "application/vnd.openxmlformats-officedocument.wordprocessingml.document": ["docx"],
    "application/vnd.ms-word.template.macroenabled.12": ["dotm"],
    "application/vnd.openxmlformats-officedocument.wordprocessingml.template": ["dotx"],
    "application/vnd.osgi.dp": ["dp"],
    "application/vnd.dpgraph": ["dpg"],
    "image/dpx": ["dpx"],
    "audio/vnd.dra": ["dra"],
    "image/dicom-rle": ["drle"],
    "text/prs.lines.tag": ["dsc"],
    "application/dssc+der": ["dssc"],
    "application/x-dtbook+xml": ["dtb"],
    "application/xml-dtd": ["dtd"],
    "audio/vnd.dts": ["dts"],
    "audio/vnd.dts.hd": ["dtshd"],
    "video/vnd.dvb.file": ["dvb"],
    "application/x-dvi": ["dvi"],
    "application/atsc-dwd+xml": ["dwd"],
    "model/vnd.dwf": ["dwf"],
    "image/vnd.dwg": ["dwg"],
    "image/vnd.dxf": ["dxf"],
    "application/vnd.spotfire.dxp": ["dxp"],
    "application/java-archive": ["ear", "jar", "war"],
    "audio/vnd.nuera.ecelp4800": ["ecelp4800"],
    "audio/vnd.nuera.ecelp7470": ["ecelp7470"],
    "audio/vnd.nuera.ecelp9600": ["ecelp9600"],
    "application/ecmascript": ["ecma"],
    "application/vnd.novadigm.edm": ["edm"],
    "application/vnd.novadigm.edx": ["edx"],
    "application/vnd.picsel": ["efif"],
    "application/vnd.pg.osasli": ["ei6"],
    "image/emf": ["emf"],
    "message/rfc822": ["eml", "mime"],
    "application/emma+xml": ["emma"],
    "application/emotionml+xml": ["emotionml"],
    "application/x-msmetafile": ["emz", "wmz"],
    "audio/vnd.digital-winds": ["eol"],
    "application/vnd.ms-fontobject": ["eot"],
    "application/epub+zip": ["epub"],
    "application/vnd.eszigno3+xml": ["es3", "et3"],
    "application/vnd.osgi.subsystem": ["esa"],
    "application/vnd.epson.esf": ["esf"],
    "text/x-setext": ["etx"],
    "application/x-eva": ["eva"],
    "application/x-envoy": ["evy"],
    "application/exi": ["exi"],
    "application/express": ["exp"],
    "image/aces": ["exr"],
    "application/vnd.novadigm.ext": ["ext"],
    "application/andrew-inset": ["ez"],
    "application/vnd.ezpix-album": ["ez2"],
    "application/vnd.ezpix-package": ["ez3"],
    "text/x-fortran": ["f", "f77", "f90", "for"],
    "video/x-f4v": ["f4v"],
    "image/vnd.fastbidsheet": ["fbs"],
    "application/vnd.adobe.formscentral.fcdt": ["fcdt"],
    "application/vnd.isac.fcs": ["fcs"],
    "application/vnd.fdf": ["fdf"],
    "application/fdt+xml": ["fdt"],
    "application/vnd.denovo.fcselayout-link": ["fe_launch"],
    "application/vnd.fujitsu.oasysgp": ["fg5"],
    "image/x-freehand": ["fh", "fh4", "fh5", "fh7", "fhc"],
    "application/x-xfig": ["fig"],
    "image/fits": ["fits"],
    "audio/x-flac": ["flac"],
    "video/x-fli": ["fli"],
    "application/vnd.micrografx.flo": ["flo"],
    "video/x-flv": ["flv"],
    "application/vnd.kde.kivio": ["flw"],
    "text/vnd.fmi.flexstor": ["flx"],
    "text/vnd.fly": ["fly"],
    "application/vnd.frogans.fnc": ["fnc"],
    "application/vnd.software602.filler.form+xml": ["fo"],
    "image/vnd.fpx": ["fpx"],
    "application/vnd.fsc.weblaunch": ["fsc"],
    "image/vnd.fst": ["fst"],
    "application/vnd.fluxtime.clip": ["ftc"],
    "application/vnd.anser-web-funds-transfer-initiation": ["fti"],
    "video/vnd.fvt": ["fvt"],
    "application/vnd.adobe.fxp": ["fxp", "fxpl"],
    "application/vnd.fuzzysheet": ["fzs"],
    "application/vnd.geoplan": ["g2w"],
    "image/g3fax": ["g3"],
    "application/vnd.geospace": ["g3w"],
    "application/vnd.groove-account": ["gac"],
    "application/x-tads": ["gam"],
    "application/rpki-ghostbusters": ["gbr"],
    "application/x-gca-compressed": ["gca"],
    "model/vnd.gdl": ["gdl"],
    "application/vnd.google-apps.document": ["gdoc"],
    "text/vnd.familysearch.gedcom": ["ged"],
    "application/vnd.dynageo": ["geo"],
    "application/geo+json": ["geojson"],
    "application/vnd.geometry-explorer": ["gex", "gre"],
    "application/vnd.geogebra.file": ["ggb"],
    "application/vnd.geogebra.tool": ["ggt"],
    "application/vnd.groove-help": ["ghf"],
    "image/gif": ["gif"],
    "application/vnd.groove-identity-message": ["gim"],
    "model/gltf-binary": ["glb"],
    "model/gltf+json": ["gltf"],
    "application/gml+xml": ["gml"],
    "application/vnd.gmx": ["gmx"],
    "application/x-gnumeric": ["gnumeric"],
    "application/vnd.flographit": ["gph"],
    "application/gpx+xml": ["gpx"],
    "application/vnd.grafeq": ["gqf", "gqs"],
    "application/srgs": ["gram"],
    "application/x-gramps-xml": ["gramps"],
    "application/vnd.groove-injector": ["grv"],
    "application/srgs+xml": ["grxml"],
    "application/x-font-ghostscript": ["gsf"],
    "application/vnd.google-apps.spreadsheet": ["gsheet"],
    "application/vnd.google-apps.presentation": ["gslides"],
    "application/x-gtar": ["gtar"],
    "application/vnd.groove-tool-message": ["gtm"],
    "model/vnd.gtw": ["gtw"],
    "text/vnd.graphviz": ["gv"],
    "application/gxf": ["gxf"],
    "application/vnd.geonext": ["gxt"],
    "application/gzip": ["gz"],
    "video/h261": ["h261"],
    "video/h263": ["h263"],
    "video/h264": ["h264"],
    "application/vnd.hal+xml": ["hal"],
    "application/vnd.hbci": ["hbci"],
    "text/x-handlebars-template": ["hbs"],
    "application/x-virtualbox-hdd": ["hdd"],
    "application/x-hdf": ["hdf"],
    "image/heic": ["heic"],
    "image/heic-sequence": ["heics"],
    "image/heif": ["heif"],
    "image/heif-sequence": ["heifs"],
    "image/hej2k": ["hej2"],
    "application/atsc-held+xml": ["held"],
    "application/hjson": ["hjson"],
    "application/winhlp": ["hlp"],
    "application/vnd.hp-hpgl": ["hpgl"],
    "application/vnd.hp-hpid": ["hpid"],
    "application/vnd.hp-hps": ["hps"],
    "application/mac-binhex40": ["hqx"],
    "image/hsj2": ["hsj2"],
    "text/x-component": ["htc"],
    "application/vnd.kenameaapp": ["htke"],
    "text/html": ["htm", "html", "shtml"],
    "application/vnd.yamaha.hv-dic": ["hvd"],
    "application/vnd.yamaha.hv-voice": ["hvp"],
    "application/vnd.yamaha.hv-script": ["hvs"],
    "application/vnd.intergeo": ["i2g"],
    "application/vnd.iccprofile": ["icc", "icm"],
    "x-conference/x-cooltalk": ["ice"],
    "image/x-icon": ["ico"],
    "text/calendar": ["ics", "ifb"],
    "image/ief": ["ief"],
    "application/vnd.shana.informed.formdata": ["ifm"],
    "model/iges": ["iges", "igs"],
    "application/vnd.igloader": ["igl"],
    "application/vnd.insors.igm": ["igm"],
    "application/vnd.micrografx.igx": ["igx"],
    "application/vnd.shana.informed.interchange": ["iif"],
    "application/vnd.accpac.simply.imp": ["imp"],
    "application/vnd.ms-ims": ["ims"],
    "application/inkml+xml": ["ink", "inkml"],
    "application/x-install-instructions": ["install"],
    "application/vnd.astraea-software.iota": ["iota"],
    "application/ipfix": ["ipfix"],
    "application/vnd.shana.informed.package": ["ipk"],
    "application/vnd.ibm.rights-management": ["irm"],
    "application/vnd.irepository.package+xml": ["irp"],
    "application/x-iso9660-image": ["iso"],
    "application/vnd.shana.informed.formtemplate": ["itp"],
    "application/its+xml": ["its"],
    "application/vnd.immervision-ivp": ["ivp"],
    "application/vnd.immervision-ivu": ["ivu"],
    "text/vnd.sun.j2me.app-descriptor": ["jad"],
    "text/jade": ["jade"],
    "application/vnd.jam": ["jam"],
    "application/x-java-archive-diff": ["jardiff"],
    "text/x-java-source": ["java"],
    "image/jphc": ["jhc"],
    "application/vnd.jisp": ["jisp"],
    "image/jls": ["jls"],
    "application/vnd.hp-jlyt": ["jlt"],
    "image/x-jng": ["jng"],
    "application/x-java-jnlp-file": ["jnlp"],
    "application/vnd.joost.joda-archive": ["joda"],
    "image/jp2": ["jp2", "jpg2"],
    "image/jpeg": ["jpe", "jpeg", "jpg"],
    "image/jpx": ["jpf", "jpx"],
    "video/jpm": ["jpgm", "jpm"],
    "video/jpeg": ["jpgv"],
    "image/jph": ["jph"],
    "text/javascript": ["js", "mjs"],
    "application/json": ["json", "map"],
    "application/json5": ["json5"],
    "application/ld+json": ["jsonld"],
    "application/jsonml+json": ["jsonml"],
    "text/jsx": ["jsx"],
    "model/jt": ["jt"],
    "image/jxr": ["jxr"],
    "image/jxra": ["jxra"],
    "image/jxrs": ["jxrs"],
    "image/jxs": ["jxs"],
    "image/jxsc": ["jxsc"],
    "image/jxsi": ["jxsi"],
    "image/jxss": ["jxss"],
    "audio/midi": ["kar", "mid", "midi", "rmi"],
    "application/vnd.kde.karbon": ["karbon"],
    "application/x-keepass2": ["kdbx"],
    "application/x-iwork-keynote-sffkey": ["key"],
    "application/vnd.kde.kformula": ["kfo"],
    "application/vnd.kidspiration": ["kia"],
    "application/vnd.google-earth.kml+xml": ["kml"],
    "application/vnd.google-earth.kmz": ["kmz"],
    "application/vnd.kinar": ["kne", "knp"],
    "application/vnd.kde.kontour": ["kon"],
    "application/vnd.kde.kpresenter": ["kpr", "kpt"],
    "application/vnd.ds-keypoint": ["kpxx"],
    "application/vnd.kde.kspread": ["ksp"],
    "application/vnd.kahootz": ["ktr", "ktz"],
    "image/ktx": ["ktx"],
    "image/ktx2": ["ktx2"],
    "application/vnd.kde.kword": ["kwd", "kwt"],
    "application/vnd.las.las+xml": ["lasxml"],
    "application/x-latex": ["latex"],
    "application/vnd.llamagraphics.life-balance.desktop": ["lbd"],
    "application/vnd.llamagraphics.life-balance.exchange+xml": ["lbe"],
    "application/vnd.hhe.lesson-player": ["les"],
    "text/less": ["less"],
    "application/lgr+xml": ["lgr"],
    "application/x-lzh-compressed": ["lha", "lzh"],
    "application/vnd.route66.link66+xml": ["link66"],
    "application/x-ms-shortcut": ["lnk"],
    "application/lost+xml": ["lostxml"],
    "application/vnd.ms-lrm": ["lrm"],
    "application/vnd.frogans.ltf": ["ltf"],
    "text/x-lua": ["lua"],
    "application/x-lua-bytecode": ["luac"],
    "audio/vnd.lucent.voice": ["lvp"],
    "application/vnd.lotus-wordpro": ["lwp"],
    "application/x-msmediaview": ["m13", "m14", "mvb"],
    "video/mpeg": ["m1v", "m2v", "mpe", "mpeg", "mpg"],
    "application/mp21": ["m21", "mp21"],
    "audio/mpeg": ["m2a", "m3a", "mp2", "mp2a", "mp3", "mpga"],
    "audio/x-mpegurl": ["m3u"],
    "application/vnd.apple.mpegurl": ["m3u8"],
    "audio/x-m4a": ["m4a"],
    "application/mp4": ["m4p", "mp4s"],
    "video/iso.segment": ["m4s"],
    "video/vnd.mpegurl": ["m4u", "mxu"],
    "video/x-m4v": ["m4v"],
    "application/mathematica": ["ma", "mb", "nb"],
    "application/mads+xml": ["mads"],
    "application/mmt-aei+xml": ["maei"],
    "application/vnd.ecowin.chart": ["mag"],
    "text/troff": ["man", "me", "ms", "roff", "t", "tr"],
    "text/markdown": ["markdown", "md"],
    "application/mathml+xml": ["mathml"],
    "application/vnd.mobius.mbk": ["mbk"],
    "application/mbox": ["mbox"],
    "application/vnd.medcalcdata": ["mc1"],
    "application/vnd.mcd": ["mcd"],
    "text/vnd.curl.mcurl": ["mcurl"],
    "application/x-msaccess": ["mdb"],
    "image/vnd.ms-modi": ["mdi"],
    "text/mdx": ["mdx"],
    "model/mesh": ["mesh", "msh", "silo"],
    "application/metalink4+xml": ["meta4"],
    "application/metalink+xml": ["metalink"],
    "application/mets+xml": ["mets"],
    "application/vnd.mfmp": ["mfm"],
    "application/rpki-manifest": ["mft"],
    "application/vnd.osgeo.mapguide.package": ["mgp"],
    "application/vnd.proteus.magazine": ["mgz"],
    "application/x-mie": ["mie"],
    "application/vnd.mif": ["mif"],
    "video/mj2": ["mj2", "mjp2"],
    "video/x-matroska": ["mk3d", "mks", "mkv"],
    "audio/x-matroska": ["mka"],
    "text/x-markdown": ["mkd"],
    "application/vnd.dolby.mlp": ["mlp"],
    "application/vnd.chipnuts.karaoke-mmd": ["mmd"],
    "application/vnd.smaf": ["mmf"],
    "text/mathml": ["mml"],
    "image/vnd.fujixerox.edmics-mmr": ["mmr"],
    "video/x-mng": ["mng"],
    "application/x-msmoney": ["mny"],
    "application/x-mobipocket-ebook": ["mobi"],
    "application/mods+xml": ["mods"],
    "video/quicktime": ["mov", "qt"],
    "video/x-sgi-movie": ["movie"],
    "video/mp4": ["mp4", "mp4v", "mpg4"],
    "audio/mp4": ["mp4a"],
    "application/vnd.mophun.certificate": ["mpc"],
    "application/dash+xml": ["mpd"],
    "application/media-policy-dataset+xml": ["mpf"],
    "application/vnd.apple.installer+xml": ["mpkg"],
    "application/vnd.blueice.multipass": ["mpm"],
    "application/vnd.mophun.application": ["mpn"],
    "application/vnd.ms-project": ["mpp", "mpt"],
    "application/vnd.ibm.minipay": ["mpy"],
    "application/vnd.mobius.mqy": ["mqy"],
    "application/marc": ["mrc"],
    "application/marcxml+xml": ["mrcx"],
    "application/mediaservercontrol+xml": ["mscml"],
    "application/vnd.fdsn.mseed": ["mseed"],
    "application/vnd.mseq": ["mseq"],
    "application/vnd.epson.msf": ["msf"],
    "application/vnd.ms-outlook": ["msg"],
    "application/msix": ["msix"],
    "application/msixbundle": ["msixbundle"],
    "application/vnd.mobius.msl": ["msl"],
    "application/vnd.muvee.style": ["msty"],
    "model/mtl": ["mtl"],
    "model/vnd.mts": ["mts"],
    "application/vnd.musician": ["mus"],
    "application/mmt-usd+xml": ["musd"],
    "application/vnd.recordare.musicxml+xml": ["musicxml"],
    "application/vnd.mapbox-vector-tile": ["mvt"],
    "application/vnd.mfer": ["mwf"],
    "application/mxf": ["mxf"],
    "application/vnd.recordare.musicxml": ["mxl"],
    "audio/mobile-xmf": ["mxmf"],
    "application/xv+xml": ["mxml", "xhvml", "xvm", "xvml"],
    "application/vnd.triscape.mxs": ["mxs"],
    "application/vnd.nokia.n-gage.symbian.install": ["n-gage"],
    "text/n3": ["n3"],
    "application/vnd.wolfram.player": ["nbp"],
    "application/x-dtbncx+xml": ["ncx"],
    "text/x-nfo": ["nfo"],
    "application/vnd.nokia.n-gage.data": ["ngdat"],
    "application/vnd.nitf": ["nitf", "ntf"],
    "application/vnd.neurolanguage.nlu": ["nlu"],
    "application/vnd.enliven": ["nml"],
    "application/vnd.noblenet-directory": ["nnd"],
    "application/vnd.noblenet-sealer": ["nns"],
    "application/vnd.noblenet-web": ["nnw"],
    "image/vnd.net-fpx": ["npx"],
    "application/n-quads": ["nq"],
    "application/x-conference": ["nsc"],
    "application/vnd.lotus-notes": ["nsf"],
    "application/n-triples": ["nt"],
    "application/x-iwork-numbers-sffnumbers": ["numbers"],
    "application/x-nzb": ["nzb"],
    "application/vnd.fujitsu.oasys2": ["oa2"],
    "application/vnd.fujitsu.oasys3": ["oa3"],
    "application/vnd.fujitsu.oasys": ["oas"],
    "application/x-msbinder": ["obd"],
    "application/vnd.openblox.game+xml": ["obgx"],
    "model/obj": ["obj"],
    "application/oda": ["oda"],
    "application/vnd.oasis.opendocument.database": ["odb"],
    "application/vnd.oasis.opendocument.chart": ["odc"],
    "application/vnd.oasis.opendocument.formula": ["odf"],
    "application/vnd.oasis.opendocument.formula-template": ["odft"],
    "application/vnd.oasis.opendocument.graphics": ["odg"],
    "application/vnd.oasis.opendocument.image": ["odi"],
    "application/vnd.oasis.opendocument.text-master": ["odm"],
    "application/vnd.oasis.opendocument.presentation": ["odp"],
    "application/vnd.oasis.opendocument.spreadsheet": ["ods"],
    "application/vnd.oasis.opendocument.text": ["odt"],
    "audio/ogg": ["oga", "ogg", "opus", "spx"],
    "model/vnd.opengex": ["ogex"],
    "video/ogg": ["ogv"],
    "application/ogg": ["ogx"],
    "application/omdoc+xml": ["omdoc"],
    "application/onenote": ["onepkg", "onetmp", "onetoc", "onetoc2"],
    "application/oebps-package+xml": ["opf"],
    "text/x-opml": ["opml"],
    "application/vnd.palm": ["oprc", "pqa"],
    "text/x-org": ["org"],
    "application/vnd.yamaha.openscoreformat": ["osf"],
    "application/vnd.yamaha.openscoreformat.osfpvg+xml": ["osfpvg"],
    "application/vnd.openstreetmap.data+xml": ["osm"],
    "application/vnd.oasis.opendocument.chart-template": ["otc"],
    "font/otf": ["otf"],
    "application/vnd.oasis.opendocument.graphics-template": ["otg"],
    "application/vnd.oasis.opendocument.text-web": ["oth"],
    "application/vnd.oasis.opendocument.image-template": ["oti"],
    "application/vnd.oasis.opendocument.presentation-template": ["otp"],
    "application/vnd.oasis.opendocument.spreadsheet-template": ["ots"],
    "application/vnd.oasis.opendocument.text-template": ["ott"],
    "application/x-virtualbox-ova": ["ova"],
    "application/x-virtualbox-ovf": ["ovf"],
    "application/rdf+xml": ["owl", "rdf"],
    "application/oxps": ["oxps"],
    "application/vnd.openofficeorg.extension": ["oxt"],
    "text/x-pascal": ["p", "pas"],
    "application/pkcs10": ["p10"],
    "application/x-pkcs12": ["p12", "pfx"],
    "application/x-pkcs7-certificates": ["p7b", "spc"],
    "application/pkcs7-mime": ["p7c", "p7m"],
    "application/x-pkcs7-certreqresp": ["p7r"],
    "application/pkcs7-signature": ["p7s"],
    "application/pkcs8": ["p8"],
    "application/x-ns-proxy-autoconfig": ["pac"],
    "application/x-iwork-pages-sffpages": ["pages"],
    "application/vnd.pawaafile": ["paw"],
    "application/vnd.powerbuilder6": ["pbd"],
    "image/x-portable-bitmap": ["pbm"],
    "application/x-font-pcf": ["pcf"],
    "application/vnd.hp-pcl": ["pcl"],
    "application/vnd.hp-pclxl": ["pclxl"],
    "image/x-pict": ["pct", "pic"],
    "application/vnd.curl.pcurl": ["pcurl"],
    "image/x-pcx": ["pcx"],
    "application/x-pilot": ["pdb"],
    "text/x-processing": ["pde"],
    "application/pdf": ["pdf"],
    "application/font-tdpfr": ["pfr"],
    "image/x-portable-graymap": ["pgm"],
    "application/x-chess-pgn": ["pgn"],
    "application/pgp-encrypted": ["pgp"],
    "application/x-httpd-php": ["php"],
    "application/pkixcmp": ["pki"],
    "application/pkix-pkipath": ["pkipath"],
    "application/vnd.apple.pkpass": ["pkpass"],
    "application/x-perl": ["pl", "pm"],
    "application/vnd.3gpp.pic-bw-large": ["plb"],
    "application/vnd.mobius.plc": ["plc"],
    "application/vnd.pocketlearn": ["plf"],
    "application/pls+xml": ["pls"],
    "application/vnd.ctc-posml": ["pml"],
    "image/png": ["png"],
    "image/x-portable-anymap": ["pnm"],
    "application/vnd.macports.portpkg": ["portpkg"],
    "application/vnd.ms-powerpoint": ["pot", "pps", "ppt"],
    "application/vnd.ms-powerpoint.template.macroenabled.12": ["potm"],
    "application/vnd.openxmlformats-officedocument.presentationml.template": ["potx"],
    "application/vnd.ms-powerpoint.addin.macroenabled.12": ["ppam"],
    "application/vnd.cups-ppd": ["ppd"],
    "image/x-portable-pixmap": ["ppm"],
    "application/vnd.ms-powerpoint.slideshow.macroenabled.12": ["ppsm"],
    "application/vnd.openxmlformats-officedocument.presentationml.slideshow": ["ppsx"],
    "application/vnd.ms-powerpoint.presentation.macroenabled.12": ["pptm"],
    "application/vnd.openxmlformats-officedocument.presentationml.presentation": [
        "pptx"
    ],
    "model/prc": ["prc"],
    "application/vnd.lotus-freelance": ["pre"],
    "application/pics-rules": ["prf"],
    "application/provenance+xml": ["provx"],
    "application/vnd.3gpp.pic-bw-small": ["psb"],
    "image/vnd.adobe.photoshop": ["psd"],
    "application/x-font-linux-psf": ["psf"],
    "application/pskc+xml": ["pskcxml"],
    "image/prs.pti": ["pti"],
    "application/vnd.pvi.ptid1": ["ptid"],
    "application/x-mspublisher": ["pub"],
    "application/vnd.3gpp.pic-bw-var": ["pvb"],
    "application/vnd.3m.post-it-notes": ["pwn"],
    "audio/vnd.ms-playready.media.pya": ["pya"],
    "model/vnd.pytha.pyox": ["pyo", "pyox"],
    "video/vnd.ms-playready.media.pyv": ["pyv"],
    "application/vnd.epson.quickanime": ["qam"],
    "application/vnd.intu.qbo": ["qbo"],
    "application/vnd.intu.qfx": ["qfx"],
    "application/vnd.publishare-delta-tree": ["qps"],
    "application/vnd.quark.quarkxpress": ["qwd", "qwt", "qxb", "qxd", "qxl", "qxt"],
    "audio/x-realaudio": ["ra"],
    "audio/x-pn-realaudio": ["ram"],
    "application/raml+yaml": ["raml"],
    "application/route-apd+xml": ["rapd"],
    "application/x-rar-compressed": ["rar"],
    "image/x-cmu-raster": ["ras"],
    "application/vnd.ipunplugged.rcprofile": ["rcprofile"],
    "application/vnd.data-vision.rdz": ["rdz"],
    "application/p2p-overlay+xml": ["relo"],
    "application/vnd.businessobjects": ["rep"],
    "application/x-dtbresource+xml": ["res"],
    "image/x-rgb": ["rgb"],
    "application/reginfo+xml": ["rif"],
    "audio/vnd.rip": ["rip"],
    "application/x-research-info-systems": ["ris"],
    "application/resource-lists+xml": ["rl"],
    "image/vnd.fujixerox.edmics-rlc": ["rlc"],
    "application/resource-lists-diff+xml": ["rld"],
    "application/vnd.rn-realmedia": ["rm"],
    "audio/x-pn-realaudio-plugin": ["rmp"],
    "application/vnd.jcp.javame.midlet-rms": ["rms"],
    "application/vnd.rn-realmedia-vbr": ["rmvb"],
    "application/relax-ng-compact-syntax": ["rnc"],
    "application/xml": ["rng", "xsd"],
    "application/rpki-roa": ["roa"],
    "application/vnd.cloanto.rp9": ["rp9"],
    "application/x-redhat-package-manager": ["rpm"],
    "application/vnd.nokia.radio-presets": ["rpss"],
    "application/vnd.nokia.radio-preset": ["rpst"],
    "application/sparql-query": ["rq"],
    "application/rls-services+xml": ["rs"],
    "application/atsc-rsat+xml": ["rsat"],
    "application/rsd+xml": ["rsd"],
    "application/urc-ressheet+xml": ["rsheet"],
    "application/rss+xml": ["rss"],
    "text/rtf": ["rtf"],
    "text/richtext": ["rtx"],
    "application/x-makeself": ["run"],
    "application/route-usd+xml": ["rusd"],
    "audio/s3m": ["s3m"],
    "application/vnd.yamaha.smaf-audio": ["saf"],
    "text/x-sass": ["sass"],
    "application/sbml+xml": ["sbml"],
    "application/vnd.ibm.secure-container": ["sc"],
    "application/x-msschedule": ["scd"],
    "application/vnd.lotus-screencam": ["scm"],
    "application/scvp-cv-request": ["scq"],
    "application/scvp-cv-response": ["scs"],
    "text/x-scss": ["scss"],
    "text/vnd.curl.scurl": ["scurl"],
    "application/vnd.stardivision.draw": ["sda"],
    "application/vnd.stardivision.calc": ["sdc"],
    "application/vnd.stardivision.impress": ["sdd"],
    "application/vnd.solent.sdkm+xml": ["sdkd", "sdkm"],
    "application/sdp": ["sdp"],
    "application/vnd.stardivision.writer": ["sdw", "vor"],
    "application/x-sea": ["sea"],
    "application/vnd.seemail": ["see"],
    "application/vnd.sema": ["sema"],
    "application/vnd.semd": ["semd"],
    "application/vnd.semf": ["semf"],
    "application/senml+xml": ["senmlx"],
    "application/sensml+xml": ["sensmlx"],
    "application/java-serialized-object": ["ser"],
    "application/set-payment-initiation": ["setpay"],
    "application/set-registration-initiation": ["setreg"],
    "application/vnd.hydrostatix.sof-data": ["sfd-hdstx"],
    "application/vnd.spotfire.sfs": ["sfs"],
    "text/x-sfv": ["sfv"],
    "image/sgi": ["sgi"],
    "application/vnd.stardivision.writer-global": ["sgl"],
    "text/sgml": ["sgm", "sgml"],
    "application/x-sh": ["sh"],
    "application/x-shar": ["shar"],
    "text/shex": ["shex"],
    "application/shf+xml": ["shf"],
    "image/x-mrsid-image": ["sid"],
    "application/sieve": ["sieve", "siv"],
    "audio/silk": ["sil"],
    "application/vnd.symbian.install": ["sis", "sisx"],
    "application/x-stuffit": ["sit"],
    "application/x-stuffitx": ["sitx"],
    "application/vnd.koan": ["skd", "skm", "skp", "skt"],
    "application/vnd.ms-powerpoint.slide.macroenabled.12": ["sldm"],
    "application/vnd.openxmlformats-officedocument.presentationml.slide": ["sldx"],
    "text/slim": ["slim", "slm"],
    "application/route-s-tsid+xml": ["sls"],
    "application/vnd.epson.salt": ["slt"],
    "application/vnd.stepmania.stepchart": ["sm"],
    "application/vnd.stardivision.math": ["smf"],
    "application/smil+xml": ["smi", "smil"],
    "video/x-smv": ["smv"],
    "application/vnd.stepmania.package": ["smzip"],
    "application/x-font-snf": ["snf"],
    "text/spdx": ["spdx"],
    "application/vnd.yamaha.smaf-phrase": ["spf"],
    "application/x-futuresplash": ["spl"],
    "text/vnd.in3d.spot": ["spot"],
    "application/scvp-vp-response": ["spp"],
    "application/scvp-vp-request": ["spq"],
    "application/x-sql": ["sql"],
    "application/x-wais-source": ["src"],
    "application/x-subrip": ["srt"],
    "application/sru+xml": ["sru"],
    "application/sparql-results+xml": ["srx"],
    "application/ssdl+xml": ["ssdl"],
    "application/vnd.kodak-descriptor": ["sse"],
    "application/vnd.epson.ssf": ["ssf"],
    "application/ssml+xml": ["ssml"],
    "application/vnd.sailingtracker.track": ["st"],
    "application/vnd.sun.xml.calc.template": ["stc"],
    "application/vnd.sun.xml.draw.template": ["std"],
    "application/vnd.wt.stf": ["stf"],
    "application/vnd.sun.xml.impress.template": ["sti"],
    "application/hyperstudio": ["stk"],
    "model/stl": ["stl"],
    "model/step+xml": ["stpx"],
    "model/step-xml+zip": ["stpxz"],
    "model/step+zip": ["stpz"],
    "application/vnd.pg.format": ["str"],
    "application/vnd.sun.xml.writer.template": ["stw"],
    "text/stylus": ["styl", "stylus"],
    "text/vnd.dvb.subtitle": ["sub"],
    "application/vnd.sus-calendar": ["sus", "susp"],
    "application/x-sv4cpio": ["sv4cpio"],
    "application/x-sv4crc": ["sv4crc"],
    "application/vnd.dvb.service": ["svc"],
    "application/vnd.svd": ["svd"],
    "image/svg+xml": ["svg", "svgz"],
    "application/x-shockwave-flash": ["swf"],
    "application/vnd.aristanetworks.swi": ["swi"],
    "application/swid+xml": ["swidtag"],
    "application/vnd.sun.xml.calc": ["sxc"],
    "application/vnd.sun.xml.draw": ["sxd"],
    "application/vnd.sun.xml.writer.global": ["sxg"],
    "application/vnd.sun.xml.impress": ["sxi"],
    "application/vnd.sun.xml.math": ["sxm"],
    "application/vnd.sun.xml.writer": ["sxw"],
    "application/x-t3vm-image": ["t3"],
    "image/t38": ["t38"],
    "application/vnd.mynfc": ["taglet"],
    "application/vnd.tao.intent-module-archive": ["tao"],
    "image/vnd.tencent.tap": ["tap"],
    "application/x-tar": ["tar"],
    "application/vnd.3gpp2.tcap": ["tcap"],
    "application/x-tcl": ["tcl", "tk"],
    "application/urc-targetdesc+xml": ["td"],
    "application/vnd.smart.teacher": ["teacher"],
    "application/tei+xml": ["tei", "teicorpus"],
    "application/x-tex": ["tex"],
    "application/x-texinfo": ["texi", "texinfo"],
    "application/thraud+xml": ["tfi"],
    "application/x-tex-tfm": ["tfm"],
    "image/tiff-fx": ["tfx"],
    "image/x-tga": ["tga"],
    "application/vnd.ms-officetheme": ["thmx"],
    "image/tiff": ["tif", "tiff"],
    "application/vnd.tmobile-livetv": ["tmo"],
    "application/toml": ["toml"],
    "application/x-bittorrent": ["torrent"],
    "application/vnd.groove-tool-template": ["tpl"],
    "application/vnd.trid.tpt": ["tpt"],
    "application/vnd.trueapp": ["tra"],
    "application/trig": ["trig"],
    "application/x-msterminal": ["trm"],
    "video/mp2t": ["ts"],
    "application/timestamped-data": ["tsd"],
    "text/tab-separated-values": ["tsv"],
    "font/collection": ["ttc"],
    "font/ttf": ["ttf"],
    "text/turtle": ["ttl"],
    "application/ttml+xml": ["ttml"],
    "application/vnd.simtech-mindmapper": ["twd", "twds"],
    "application/vnd.genomatix.tuxedo": ["txd"],
    "application/vnd.mobius.txf": ["txf"],
    "model/u3d": ["u3d"],
    "message/global-delivery-status": ["u8dsn"],
    "message/global-headers": ["u8hdr"],
    "message/global-disposition-notification": ["u8mdn"],
    "message/global": ["u8msg"],
    "application/ubjson": ["ubj"],
    "application/vnd.ufdl": ["ufd", "ufdl"],
    "application/x-glulx": ["ulx"],
    "application/vnd.umajin": ["umj"],
    "application/vnd.unity": ["unityweb"],
    "application/vnd.uoml+xml": ["uo", "uoml"],
    "text/uri-list": ["uri", "uris", "urls"],
    "model/vnd.usda": ["usda"],
    "model/vnd.usdz+zip": ["usdz"],
    "application/x-ustar": ["ustar"],
    "application/vnd.uiq.theme": ["utz"],
    "text/x-uuencode": ["uu"],
    "audio/vnd.dece.audio": ["uva", "uvva"],
    "application/vnd.dece.data": ["uvd", "uvf", "uvvd", "uvvf"],
    "image/vnd.dece.graphic": ["uvg", "uvi", "uvvg", "uvvi"],
    "video/vnd.dece.hd": ["uvh", "uvvh"],
    "video/vnd.dece.mobile": ["uvm", "uvvm"],
    "video/vnd.dece.pd": ["uvp", "uvvp"],
    "video/vnd.dece.sd": ["uvs", "uvvs"],
    "application/vnd.dece.ttml+xml": ["uvt", "uvvt"],
    "video/vnd.uvvu.mp4": ["uvu", "uvvu"],
    "video/vnd.dece.video": ["uvv", "uvvv"],
    "application/vnd.dece.unspecified": ["uvvx", "uvx"],
    "application/vnd.dece.zip": ["uvvz", "uvz"],
    "application/x-virtualbox-vbox": ["vbox"],
    "application/x-virtualbox-vbox-extpack": ["vbox-extpack"],
    "text/vcard": ["vcard"],
    "application/x-cdlink": ["vcd"],
    "text/x-vcard": ["vcf"],
    "application/vnd.groove-vcard": ["vcg"],
    "text/x-vcalendar": ["vcs"],
    "application/vnd.vcx": ["vcx"],
    "application/x-virtualbox-vdi": ["vdi"],
    "model/vnd.sap.vds": ["vds"],
    "application/x-virtualbox-vhd": ["vhd"],
    "application/vnd.visionary": ["vis"],
    "video/vnd.vivo": ["viv"],
    "application/x-virtualbox-vmdk": ["vmdk"],
    "video/x-ms-vob": ["vob"],
    "model/vrml": ["vrml", "wrl"],
    "application/vnd.visio": ["vsd", "vss", "vst", "vsw"],
    "application/vnd.vsf": ["vsf"],
    "image/vnd.valve.source.texture": ["vtf"],
    "text/vtt": ["vtt"],
    "model/vnd.vtu": ["vtu"],
    "application/voicexml+xml": ["vxml"],
    "application/x-doom": ["wad"],
    "application/vnd.sun.wadl+xml": ["wadl"],
    "application/wasm": ["wasm"],
    "audio/x-wav": ["wav"],
    "audio/x-ms-wax": ["wax"],
    "image/vnd.wap.wbmp": ["wbmp"],
    "application/vnd.criticaltools.wbs+xml": ["wbs"],
    "application/vnd.wap.wbxml": ["wbxml"],
    "application/vnd.ms-works": ["wcm", "wdb", "wks", "wps"],
    "image/vnd.ms-photo": ["wdp"],
    "audio/webm": ["weba"],
    "application/x-web-app-manifest+json": ["webapp"],
    "video/webm": ["webm"],
    "application/manifest+json": ["webmanifest"],
    "image/webp": ["webp"],
    "application/vnd.pmi.widget": ["wg"],
    "text/wgsl": ["wgsl"],
    "application/widget": ["wgt"],
    "application/watcherinfo+xml": ["wif"],
    "video/x-ms-wm": ["wm"],
    "audio/x-ms-wma": ["wma"],
    "application/x-ms-wmd": ["wmd"],
    "image/wmf": ["wmf"],
    "text/vnd.wap.wml": ["wml"],
    "application/vnd.wap.wmlc": ["wmlc"],
    "text/vnd.wap.wmlscript": ["wmls"],
    "application/vnd.wap.wmlscriptc": ["wmlsc"],
    "video/x-ms-wmv": ["wmv"],
    "video/x-ms-wmx": ["wmx"],
    "font/woff": ["woff"],
    "font/woff2": ["woff2"],
    "application/vnd.wordperfect": ["wpd"],
    "application/vnd.ms-wpl": ["wpl"],
    "application/vnd.wqd": ["wqd"],
    "application/x-mswrite": ["wri"],
    "message/vnd.wfa.wsc": ["wsc"],
    "application/wsdl+xml": ["wsdl"],
    "application/wspolicy+xml": ["wspolicy"],
    "application/vnd.webturbo": ["wtb"],
    "video/x-ms-wvx": ["wvx"],
    "model/x3d+xml": ["x3d", "x3dz"],
    "model/x3d+fastinfoset": ["x3db"],
    "model/x3d+binary": ["x3dbz"],
    "model/x3d-vrml": ["x3dv"],
    "model/x3d+vrml": ["x3dvz"],
    "model/vnd.parasolid.transmit.binary": ["x_b"],
    "model/vnd.parasolid.transmit.text": ["x_t"],
    "application/xaml+xml": ["xaml"],
    "application/x-silverlight-app": ["xap"],
    "application/vnd.xara": ["xar"],
    "application/xcap-att+xml": ["xav"],
    "application/x-ms-xbap": ["xbap"],
    "application/vnd.fujixerox.docuworks.binder": ["xbd"],
    "image/x-xbitmap": ["xbm"],
    "application/xcap-caps+xml": ["xca"],
    "application/calendar+xml": ["xcs"],
    "application/xcap-diff+xml": ["xdf"],
    "application/vnd.syncml.dm+xml": ["xdm"],
    "application/vnd.adobe.xdp+xml": ["xdp"],
    "application/dssc+xml": ["xdssc"],
    "application/vnd.fujixerox.docuworks": ["xdw"],
    "application/xcap-el+xml": ["xel"],
    "application/xenc+xml": ["xenc"],
    "application/patch-ops-error+xml": ["xer"],
    "application/xfdf": ["xfdf"],
    "application/vnd.xfdl": ["xfdl"],
    "application/xhtml+xml": ["xht", "xhtml"],
    "application/vnd.pwg-xhtml-print+xml": ["xhtm"],
    "image/vnd.xiff": ["xif"],
    "application/vnd.ms-excel": ["xla", "xlc", "xlm", "xls", "xlt", "xlw"],
    "application/vnd.ms-excel.addin.macroenabled.12": ["xlam"],
    "application/xliff+xml": ["xlf"],
    "application/vnd.ms-excel.sheet.binary.macroenabled.12": ["xlsb"],
    "application/vnd.ms-excel.sheet.macroenabled.12": ["xlsm"],
    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": ["xlsx"],
    "application/vnd.ms-excel.template.macroenabled.12": ["xltm"],
    "application/vnd.openxmlformats-officedocument.spreadsheetml.template": ["xltx"],
    "audio/xm": ["xm"],
    "text/xml": ["xml"],
    "application/xcap-ns+xml": ["xns"],
    "application/vnd.olpc-sugar": ["xo"],
    "application/xop+xml": ["xop"],
    "application/x-xpinstall": ["xpi"],
    "application/xproc+xml": ["xpl"],
    "image/x-xpixmap": ["xpm"],
    "application/vnd.is-xpr": ["xpr"],
    "application/vnd.ms-xpsdocument": ["xps"],
    "application/vnd.intercon.formnet": ["xpw", "xpx"],
    "application/prs.xsf+xml": ["xsf"],
    "application/xslt+xml": ["xsl", "xslt"],
    "application/vnd.syncml+xml": ["xsm"],
    "application/xspf+xml": ["xspf"],
    "application/vnd.mozilla.xul+xml": ["xul"],
    "image/x-xwindowdump": ["xwd"],
    "chemical/x-xyz": ["xyz"],
    "application/x-xz": ["xz"],
    "text/yaml": ["yaml", "yml"],
    "application/yang": ["yang"],
    "application/yin+xml": ["yin"],
    "text/x-suse-ymp": ["ymp"],
    "application/x-zmachine": ["z1", "z2", "z3", "z4", "z5", "z6", "z7", "z8"],
    "application/vnd.zzazz.deck+xml": ["zaz"],
    "application/zip": ["zip"],
    "application/vnd.zul": ["zir", "zirz"],
    "application/vnd.handheld-entertainment+xml": ["zmm"],
    "inode/x-empty": [],
}

checkim = get_filepath("file.exe")
startupinfo = subprocess.STARTUPINFO()
creationflags = 0 | subprocess.CREATE_NO_WINDOW
startupinfo.wShowWindow = subprocess.SW_HIDE
invisibledict = {
    "startupinfo": startupinfo,
    "creationflags": creationflags,
}


def filelist2df(flist):
    import importlib

    pd = importlib.import_module("pandas")

    def get_item_from_iter(vara, indi):
        try:
            return vara[indi]
        except Exception:
            return pd.NA

    columnname = 0
    df = pd.concat([pd.DataFrame(q) for q in flist])
    maxlenaal = df[columnname].dropna().copy()
    maxlenaal = maxlenaal.loc[
        maxlenaal.apply(lambda x: not isinstance(x, (int, float, bool)))
    ]
    maxlena = maxlenaal.apply(len).max()
    collectionofdf = []
    collectionofdfcol = []
    for co in range(maxlena):
        collectionofdf.append(
            df[columnname].apply(lambda x: get_item_from_iter(x, co)).copy()
        )
        collectionofdfcol.append(str(columnname) + "_" + str(co))
    dfe = (
        pd.concat(collectionofdf, axis=1, ignore_index=True)
        .fillna(pd.NA)
        .reset_index(drop=True)
    )
    dfe.columns = [
        "aa_filename",
        "aa_mime",
        "aa_encoding",
        "aa_possible_extensions",
        "aa_possible_filenames",
    ]

    return dfe


def get_file_list_from_folders(path, maxsubfolders=-1):
    return [x.path for x in get_folder_file_complete_path(path, maxsubfolders)]


def guess_filetypes(
    files_folders: list | str,
    maxsubfolders: int = -1,
    pandas_dataframe: bool = False,
    verbose: bool = True,
):
    r"""
    Guesses the file types of the provided files and folders. It uses the binaries from https://github.com/julian-r/file-windows/releases
    (File and Libmagic build with Visual Studio) - They are included in this package


    Args:
        files_folders (list or str): A list of file/folder paths or a single file/folder path.
        maxsubfolders (int, optional): Maximum number of subfolders to scan. Default is -1, which means no limit.
        pandas_dataframe (bool, optional): Determines if the results should be returned as a pandas DataFrame.
                                           Requires pandas to be installed. Default is False.
        verbose (bool, optional): Determines if verbose output should be displayed. Default is True.

    Returns:
        list or pd.DataFrame: A list of file type information for each file or a pandas DataFrame if pandas_dataframe is True.

    Raises:
        Exception: If pandas is not installed and pandas_dataframe is set to True.

    Example:
result_list = guess_filetypes(
    files_folders=[
        r"C:\Users\hansc\Pictures\fastcpy",  # png file without ending
        r"C:\Users\hansc\Pictures\fastcpy - Copy.png",  # an actual png file, to check if files with the correct ending are ignored
        r"C:\Users\hansc\Pictures\cppcomp.jpg",  # a .txt file with the wrong ending
        r"E:\destinationcopytemp5",  # internet cache files - whole folder will be scanned
    ],
    maxsubfolders=-1,  # if you want to limit the number of subfolders to scan, -1 means no limit
    pandas_dataframe=False,  # return the results as a pd.DataFrame (pandas must be installed)
    verbose=True,  # visual output
)

result_df = guess_filetypes(
    files_folders=[
        r"C:\Users\hansc\Pictures\fastcpy",  # png file without ending
        r"C:\Users\hansc\Pictures\fastcpy - Copy.png",  # an actual png file, to check if files with the correct ending are ignored
        r"C:\Users\hansc\Pictures\cppcomp.jpg",  # a .txt file with the wrong ending
        r"E:\destinationcopytemp5",  # internet cache files - whole folder will be scanned
    ],
    maxsubfolders=-1,
    pandas_dataframe=True,
    verbose=True,)
    output:
    [[['C:\\Users\\hansc\\Pictures\\fastcpy', 'image/png', 'charset=binary', ('png',), ('C:\\Users\\hansc\\Pictures\\fastcpy.png',)]]]
    [[['C:\\Users\\hansc\\Pictures\\fastcpy - Copy.png', 'image/png', 'charset=binary', ('png',), ('C:\\Users\\hansc\\Pictures\\fastcpy - Copy.png',)]]]
    [[['C:\\Users\\hansc\\Pictures\\cppcomp.jpg', 'text/plain', 'charset=us-ascii', ('conf', 'def', 'in', 'ini', 'list', 'log', 'text', 'txt'), ('C:\\Users\\hansc\\Pictures\\cppcomp.jpg.conf', 'C:\\Users\\hansc\\Pictures\\cppcomp.jpg.def', 'C:\\Users\\hansc\\Pictures\\cppcomp.jpg.in', 'C:\\Users\\hansc\\Pictures\\cppcomp.jpg.ini', 'C:\\Users\\hansc\\Pictures\\cppcomp.jpg.list', 'C:\\Users\\hansc\\Pictures\\cppcomp.jpg.log', 'C:\\Users\\hansc\\Pictures\\cppcomp.jpg.text', 'C:\\Users\\hansc\\Pictures\\cppcomp.jpg.txt')]]]
    [[['E:\\destinationcopytemp5\\00000000__2023_05_13_22_33_46\\Users\\hansc\\AppData\\Local\\Temp\\RBX-64BA6ED6.log', 'text/plain', 'charset=us-ascii', ('conf', 'def', 'in', 'ini', 'list', 'log', 'text', 'txt'), ('E:\\destinationcopytemp5\\00000000__2023_05_13_22_33_46\\Users\\hansc\\AppData\\Local\\Temp\\RBX-64BA6ED6.log',)]]]
    [[['E:\\destinationcopytemp5\\00000000__2023_05_13_22_33_46\\Users\\hansc\\AppData\\Local\\Temp\\RBX-DF39BC9A.log', 'text/plain', 'charset=us-ascii', ('conf', 'def', 'in', 'ini', 'list', 'log', 'text', 'txt'), ('E:\\destinationcopytemp5\\00000000__2023_05_13_22_33_46\\Users\\hansc\\AppData\\Local\\Temp\\RBX-DF39BC9A.log',)]]]


                                                                                                 aa_filename     aa_mime       aa_encoding                      aa_possible_extensions                                                                                                                                                                                                                                                                                                                       aa_possible_filenames
    0                                                                        C:\Users\hansc\Pictures\fastcpy   image/png    charset=binary                                      (png,)                                                                                                                                                                                                                                                                                                      (C:\Users\hansc\Pictures\fastcpy.png,)
    1                                                             C:\Users\hansc\Pictures\fastcpy - Copy.png   image/png    charset=binary                                      (png,)                                                                                                                                                                                                                                                                                               (C:\Users\hansc\Pictures\fastcpy - Copy.png,)
    2                                                                    C:\Users\hansc\Pictures\cppcomp.jpg  text/plain  charset=us-ascii  (conf, def, in, ini, list, log, text, txt)  (C:\Users\hansc\Pictures\cppcomp.jpg.conf, C:\Users\hansc\Pictures\cppcomp.jpg.def, C:\Users\hansc\Pictures\cppcomp.jpg.in, C:\Users\hansc\Pictures\cppcomp.jpg.ini, C:\Users\hansc\Pictures\cppcomp.jpg.list, C:\Users\hansc\Pictures\cppcomp.jpg.log, C:\Users\hansc\Pictures\cppcomp.jpg.text, C:\Users\hansc\Pictures\cppcomp.jpg.txt)
    3  E:\destinationcopytemp5\00000000__2023_05_13_22_33_46\Users\hansc\AppData\Local\Temp\RBX-64BA6ED6.log  text/plain  charset=us-ascii  (conf, def, in, ini, list, log, text, txt)                                                                                                                                                                                                                                    (E:\destinationcopytemp5\00000000__2023_05_13_22_33_46\Users\hansc\AppData\Local\Temp\RBX-64BA6ED6.log,)
    4  E:\destinationcopytemp5\00000000__2023_05_13_22_33_46\Users\hansc\AppData\Local\Temp\RBX-DF39BC9A.log  text/plain  charset=us-ascii  (conf, def, in, ini, list, log, text, txt)                                                                                                                                                                                                                                    (E:\destinationcopytemp5\00000000__2023_05_13_22_33_46\Users\hansc\AppData\Local\Temp\RBX-DF39BC9A.log,)



    """
    if not isinstance(files_folders, list):
        files_folders = [files_folders]
    allfi = []
    for f in files_folders:
        if os.path.isdir(f) or os.path.ismount(f):
            allfi.extend(get_file_list_from_folders(f, maxsubfolders=maxsubfolders))
        if os.path.isfile(f):
            allfi.append(f)

    allresults2 = []
    allresults = []

    for fi in allfi:
        command = [
            checkim,
            fi,
            "--mime",
            "--separator",
            "ÇÇÇÇ",
        ]
        p = subprocess.run(command, capture_output=True, shell=True, **invisibledict)
        resu2 = re.findall(
            rb"(^[^\n\r]+)\xc7\xc7\xc7\xc7\s+([^\n\r]+);\s+([^\n\r]+)", p.stdout
        )
        allresults.append(resu2)

        allresults2.append(
            [
                [
                    [
                        a := x[0].decode("utf-8"),
                        g := x[1].decode("utf-8"),
                        x[2].decode("utf-8"),
                        m := tuple(mimesreversed.get(g, [])),
                        tuple([a + "." + mm for mm in m])
                        if not pathlib.Path(a).suffix.strip(".").lower() in m
                        else (a,),
                    ]
                    for x in resu2
                ]
            ]
        )
        if verbose:
            print(allresults2[-1])
    if pandas_dataframe:
        try:
            return filelist2df(allresults2)
        except Exception as fe:
            print(fe)
            print(
                "pandas could not be found. (pip install pandas)\nthe results will be returned as a list"
            )
    return allresults2
